<?php

namespace Classifieds\Chat\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Classifieds\Chat\Http\Requests\ChatRequest;
use Classifieds\Chat\Interfaces\ChatRepositoryInterface;
use Classifieds\Chat\Models\Chat;

/**
 * Resource controller class for chat.
 */
class ChatResourceController extends BaseController
{

    /**
     * Initialize chat resource controller.
     *
     * @param type ChatRepositoryInterface $chat
     *
     * @return null
     */
    public function __construct(ChatRepositoryInterface $chat)
    {
        parent::__construct();
        $this->repository = $chat;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Classifieds\Chat\Repositories\Criteria\ChatResourceCriteria::class);
    }

    /**
     * Display a list of chat.
     *
     * @return Response
     */
    public function index(ChatRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Classifieds\Chat\Repositories\Presenter\ChatPresenter::class)
                ->$function();
        }

        $chats = $this->repository->paginate();

        return $this->response->setMetaTitle(trans('chat::chat.names'))
            ->view('chat::chat.index', true)
            ->data(compact('chats', 'view'))
            ->output();
    }

    /**
     * Display chat.
     *
     * @param Request $request
     * @param Model   $chat
     *
     * @return Response
     */
    public function show(ChatRequest $request, Chat $chat)
    {

        if ($chat->exists) {
            $view = 'chat::chat.show';
        } else {
            $view = 'chat::chat.new';
        }

        return $this->response->setMetaTitle(trans('app.view') . ' ' . trans('chat::chat.name'))
            ->data(compact('chat'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new chat.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(ChatRequest $request)
    {

        $chat = $this->repository->newInstance([]);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('chat::chat.name')) 
            ->view('chat::chat.create', true) 
            ->data(compact('chat'))
            ->output();
    }

    /**
     * Create new chat.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(ChatRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $chat                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('chat::chat.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('chat/chat/' . $chat->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/chat/chat'))
                ->redirect();
        }

    }

    /**
     * Show chat for editing.
     *
     * @param Request $request
     * @param Model   $chat
     *
     * @return Response
     */
    public function edit(ChatRequest $request, Chat $chat)
    {
        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('chat::chat.name'))
            ->view('chat::chat.edit', true)
            ->data(compact('chat'))
            ->output();
    }

    /**
     * Update the chat.
     *
     * @param Request $request
     * @param Model   $chat
     *
     * @return Response
     */
    public function update(ChatRequest $request, Chat $chat)
    {
        try {
            $attributes = $request->all();

            $chat->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('chat::chat.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('chat/chat/' . $chat->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('chat/chat/' . $chat->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the chat.
     *
     * @param Model   $chat
     *
     * @return Response
     */
    public function destroy(ChatRequest $request, Chat $chat)
    {
        try {

            $chat->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('chat::chat.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('chat/chat/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('chat/chat/' . $chat->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple chat.
     *
     * @param Model   $chat
     *
     * @return Response
     */
    public function delete(ChatRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('chat::chat.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('chat/chat'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/chat/chat'))
                ->redirect();
        }

    }

    /**
     * Restore deleted chats.
     *
     * @param Model   $chat
     *
     * @return Response
     */
    public function restore(ChatRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('chat::chat.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/chat/chat'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/chat/chat/'))
                ->redirect();
        }

    }

}
